﻿using System;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;

namespace Amplitude.AI.Core.Brain.Analysis.Jobs.GameNodeBased
{
    public class ImportanceScoring : AnalysisJob
    {
        public ImportanceScoring()
        {
            base.WaitForOtherJob.Add(typeof(ColonizationQualityScoring));
            base.WaitForOtherJob.Add(typeof(GameNodeStrategicScore));
        }

        protected override void Compute()
        {
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            if (service == null)
            {
                return;
            }
            GalaxySnapshot galaxySnapshot = service.GalaxySnapshot;
            if (!(base.EmpireSnapshot is MajorEmpireSnapshot))
            {
                return;
            }
            HeuristicFloat heuristicFloat = new HeuristicFloat(1f, "");
            for (int i = 0; i < galaxySnapshot.GameNodes.Length; i++)
            {
                NodeData analysisData = galaxySnapshot.GameNodes[i].GetAnalysisData<NodeData>(base.EmpireIndex);
                HeuristicFloat heuristicFloat2 = default(HeuristicFloat);
                heuristicFloat2.Add(analysisData.StrategicScore, "strategic score");
                heuristicFloat2.Multiply(5f, "constant");
                heuristicFloat2.Add(analysisData.ColonizationScore, "colonisation score");
                heuristicFloat2.Add(analysisData.EconomicScore, "economic score");
                analysisData.Importance = heuristicFloat2;
                if (heuristicFloat2.Value > heuristicFloat.Value && galaxySnapshot.GameNodes[i].HasColonyOf(base.EmpireSnapshot, false, -1, true))
                {
                    heuristicFloat = heuristicFloat2;
                }
            }
            base.EmpireSnapshot.AIPlayer.AnalysisDataHolder.MyEmpireData.MaxSystemImportanceScore = heuristicFloat;
        }
    }
}
