﻿using System;
using System.Collections.Generic;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Decision.Resource;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Colony;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Politic;

namespace Amplitude.AI.Core.Brain.Analysis.Jobs.EmpireBased.ResourceNeeds
{
    public class ResourceNeed : AnalysisJob
    {
        protected override void Compute()
        {
            MyEmpireData myEmpireData = base.EmpireSnapshot.Owner.AnalysisDataHolder.MyEmpireData;
            MajorEmpireSnapshot majorEmpireSnapshot = base.EmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot == null)
            {
                return;
            }
            for (int i = 0; i < majorEmpireSnapshot.ColonizedStarSystems.Count; i++)
            {
                ColonizedStarSystemSnapshot colonizedStarSystemSnapshot = majorEmpireSnapshot.ColonizedStarSystems[i];
                if (colonizedStarSystemSnapshot.State == ColonizedStarSystemSnapshot.StarSystemState.Colony)
                {
                    colonizedStarSystemSnapshot.StarSystemNode.GetAnalysisData<NodeData>(base.EmpireIndex).IndustryNeed = this.ComputeFlowAllocatorScore(colonizedStarSystemSnapshot, colonizedStarSystemSnapshot.SystemProductionNet, "SystemProduction");
                }
            }
            HeuristicFloat dustNeed = this.ComputeStockAllocatorScore(majorEmpireSnapshot, "EmpireMoney");
            float empireMoneyNet = base.EmpireSnapshot.EmpireMoneyNet;
            if (empireMoneyNet < 0f)
            {
                dustNeed.Boost(0.5f, "Income is negative");
                float num = -base.EmpireSnapshot.EmpireMoneyStock / empireMoneyNet;
                if (num < 5f)
                {
                    dustNeed.Boost(0.5f, "Less than 5 turns until bankruptcy");
                }
                else if (num < 15f)
                {
                    dustNeed.Boost(0.25f, "Less than 15 turns until bankruptcy");
                }
            }
            if (base.EmpireSnapshot.HasPersonality("Mercantile"))
            {
                dustNeed.Boost(0.2f, "'Mercantile' personality");
            }
            myEmpireData.DustNeed = dustNeed;
            HeuristicFloat prestigeNeed = this.ComputeStockAllocatorScore(majorEmpireSnapshot, "EmpireEmpirePoint");
            float empireEmpirePointNet = base.EmpireSnapshot.EmpireEmpirePointNet;
            if (empireEmpirePointNet < 0f)
            {
                prestigeNeed.Boost(0.5f, "Income is negative");
                float num2 = -base.EmpireSnapshot.EmpireEmpirePointStock / empireEmpirePointNet;
                if (num2 < 5f)
                {
                    prestigeNeed.Boost(0.5f, "Less than 5 turns until bankruptcy");
                }
                else if (num2 < 15f)
                {
                    prestigeNeed.Boost(0.25f, "Less than 15 turns until bankruptcy");
                }
            }
            if (base.EmpireSnapshot.HasPersonality("Pacifist"))
            {
                prestigeNeed.Boost(0.1f, "'Pacifist' personality");
            }
            if (base.EmpireSnapshot.HasPersonality("Militarist"))
            {
                prestigeNeed.Boost(0.1f, "'Militarist' personality");
            }
            if (base.EmpireSnapshot.CanBuyoutWithEmpirePoint)
            {
                prestigeNeed.Boost(0.3f, "can buyout by influence");
            }
            int num3 = 0;
            if (majorEmpireSnapshot.SenatSnapshot != null && majorEmpireSnapshot.SenatSnapshot.ActiveLaws != null)
            {
                using (List<ActiveLawSnapshot>.Enumerator enumerator = majorEmpireSnapshot.SenatSnapshot.ActiveLaws.GetEnumerator())
                {
                    while (enumerator.MoveNext())
                    {
                        if (enumerator.Current.LawSnapshot.UpkeepPerPopulation > 0f)
                        {
                            num3++;
                        }
                    }
                }
            }
            prestigeNeed.Boost((float)num3 * 0.1f, "We are keeping laws.");
            int num4 = 0;
            foreach (DiplomaticRelationSnapshot diplomaticRelationSnapshot in majorEmpireSnapshot.DiplomaticRelations)
            {
                if (diplomaticRelationSnapshot.DiplomaticAbilitiesWithUpkeep != null)
                {
                    num4 += diplomaticRelationSnapshot.DiplomaticAbilitiesWithUpkeep.Length;
                }
            }
            prestigeNeed.Boost((float)num4 * 0.1f, "We are keeping diplomatic abilities.");
            myEmpireData.PrestigeNeed = prestigeNeed;
            HeuristicFloat scienceNeed = this.ComputeFlowAllocatorScore(majorEmpireSnapshot, majorEmpireSnapshot.EmpireResearchNet, "EmpireResearch");
            if (base.EmpireSnapshot.HasPersonality("Innovative"))
            {
                scienceNeed.Boost(0.2f, "'Innovative' personality");
            }
            myEmpireData.ScienceNeed = scienceNeed;
            HeuristicFloat honorNeed;
            if (!majorEmpireSnapshot.UseHonor)
            {
                honorNeed = new HeuristicFloat(0f, "This faction does not use Honor");
            }
            else
            {
                honorNeed = new HeuristicFloat((100f - majorEmpireSnapshot.EmpireHonorStock) / 100f, "Linear motivation for Honor gain");
                honorNeed.Add(0.25f, "Constant boost to try reaching the way of the obsidian eagle");
                honorNeed.Clamp01();
                honorNeed.Smoothstep(2);
            }
            myEmpireData.HonorNeed = honorNeed;
            if (majorEmpireSnapshot.MaximumEmpireProcessingPowerStock <= 0f)
            {
                myEmpireData.ProcessingPowerNeed = new HeuristicFloat(1f, "");
                return;
            }
            HeuristicFloat processingPowerNeed = default(HeuristicFloat);
            float operand = myEmpireData.DefensiveHackingNeeds + myEmpireData.OffensiveHackingNeeds + myEmpireData.EmergencyHackingNeeds + myEmpireData.BeaconHackingNeeds;
            processingPowerNeed.Add(operand, "My total hacking needs");
            processingPowerNeed.Divide(majorEmpireSnapshot.MaximumEmpireProcessingPowerStock, "Dividing by maximum empire processing poer stock");
            processingPowerNeed.Multiply(0.4f, "Factor scaling the need to a decent value");
            processingPowerNeed.Clamp01();
            processingPowerNeed.Smoothstep(2);
            myEmpireData.ProcessingPowerNeed = processingPowerNeed;
        }

        private HeuristicFloat ComputeStockAllocatorScore(MajorEmpireSnapshot majorEmpire, string resourceName)
        {
            HeuristicFloat result = default(HeuristicFloat);
            ResourceAllocator_Resource resourceAllocator_Resource = majorEmpire.GetResourceAllocatorFor(resourceName) as ResourceAllocator_Resource;
            if (resourceAllocator_Resource != null && resourceAllocator_Resource.FailedAllocations.Count != 0)
            {
                result.Add(resourceAllocator_Resource.FailedAllocations[0].Motivation, "Failed allocation of {0}", resourceAllocator_Resource.FailedAllocations[0].LocalizedName);
            }
            return result;
        }

        private HeuristicFloat ComputeFlowAllocatorScore(ResourceDataProvider dataprovider, float net, string resourceName)
        {
            HeuristicFloat result = new HeuristicFloat(0f, "");
            if (net <= 0f)
            {
                result.Add(1f, "{0} production is negative or null", resourceName);
                return result;
            }
            ResourceAllocator_Resource resourceAllocator_Resource = dataprovider.GetResourceAllocatorFor(resourceName) as ResourceAllocator_Resource;
            if (resourceAllocator_Resource != null && resourceAllocator_Resource.LastCost != 0f && !float.IsInfinity(resourceAllocator_Resource.LastCost))
            {
                float operand = (net > 0f) ? (resourceAllocator_Resource.LastCost / net) : 1000f;
                result.Add(operand, "Nb turns to do one operation");
                if (resourceName == "SystemProduction")
                {
                    result.Divide((float)Duration.FromNormalSpeed(10).Value, "Max allowed duration for one operation");
                }
                else if (resourceName == "EmpireResearch")
                {
                    int num = 10;
                    int num2 = base.EmpireSnapshot.ResearchedTechnologies / num;
                    result.Divide((float)Duration.FromNormalSpeed(10 + num2 * 2).Value, "Max allowed duration for one operation  [Era" + (num2 + 1).ToString() + "]");
                }
                result.Clamp01();
            }
            return result;
        }

        public ResourceNeed()
        {
        }
    }
}
