﻿using System;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Modules;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Galaxy;

namespace Amplitude.AI.Core.Brain.Analysis.Jobs.EmpireBased
{
    public class ComputeWantedSystemCount : AnalysisJob
    {
        public ComputeWantedSystemCount()
        {
            base.WaitForOtherJob.Add(typeof(ComputeDepletionRatio));
        }

        protected override void Compute()
        {
            MyEmpireData myEmpireData = base.EmpireSnapshot.Owner.AnalysisDataHolder.MyEmpireData;
            MajorEmpireSnapshot majorEmpireSnapshot = base.EmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot == null)
            {
                return;
            }
            if (this.turn.Length != this.factorForTurn.Length)
            {
                Diagnostics.LogError("Invalid length between Turn and Factor... J-L what have you done!?!?!?!");
                return;
            }
            HeuristicFloat operand = this.ComputeExpectedPercentBasedOnTurn();
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            if (service == null)
            {
                return;
            }
            int num3 = service.GalaxySnapshot.GameNodes.Length;
            float num2 = 0f;
            for (int i = 0; i < service.GalaxySnapshot.GameNodes.Length; i++)
            {
                StarSystemNodeSnapshot starSystemNodeSnapshot = service.GalaxySnapshot.GameNodes[i] as StarSystemNodeSnapshot;
                if (starSystemNodeSnapshot != null && starSystemNodeSnapshot.HasVisiblePlanets(base.EmpireIndex))
                {
                    num2 += 1f;
                }
            }
            HeuristicFloat operand2 = default(HeuristicFloat);
            operand2.Add((float)base.EmpireSnapshot.GetColonyCount(), "The current number of systems that we control");
            operand2.Multiply(majorEmpireSnapshot.Owner.AnalysisDataHolder.MyEmpireData.GlobalDepletionRatio, "The overall depletion of these systems");
            HeuristicFloat wantedSystemCount = default(HeuristicFloat);
            wantedSystemCount.Add(operand, "Percent of the galaxy to conquer");
            wantedSystemCount.Multiply(num2, "Number of system (more or less colonizable) in the galaxy");
            wantedSystemCount.Add(operand2, "Find replacement systems for our population currently living in depleted systems");
            myEmpireData.WantedSystemCount = wantedSystemCount;
        }

        private HeuristicFloat ComputeExpectedPercentBasedOnTurn()
        {
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            int num = service.GameSnapshot.Turn;
            bool flag = false;
            HeuristicFloat operand = default(HeuristicFloat);
            for (int i = 0; i < this.turn.Length; i++)
            {
                if (num == this.turn[i])
                {
                    operand.Add(this.factorForTurn[i], "CurrentTurn ({1}) == Turn[{0}]", i, num);
                    flag = true;
                    break;
                }
                if (num < this.turn[i])
                {
                    float num2 = (float)this.turn[i - 1];
                    float num3 = (float)this.turn[i];
                    float operand2 = ((float)num - num2) / (num3 - num2);
                    float operand3 = this.factorForTurn[i - 1];
                    float operand4 = this.factorForTurn[i];
                    HeuristicFloat operand5 = default(HeuristicFloat);
                    operand5.Add(operand3, "Previous factor.");
                    operand5.Subtract(operand4, "Next factor.");
                    operand5.Multiply(operand2, "Turn ratio between {0} and {1}", num2, num3);
                    operand.Add(operand3, "Previous factor at Turn {0}.", num2);
                    operand.Subtract(operand5, "Current turn = {0}, next step is at {1}.", num, num3);
                    flag = true;
                    break;
                }
            }
            if (!flag)
            {
                operand.Add(this.factorForTurn[this.factorForTurn.Length - 1], "Over the last step.");
            }
            float num4 = 0f;
            for (int j = 0; j < service.GameSnapshot.EmpireSnapshots.Length; j++)
            {
                if (service.GameSnapshot.EmpireSnapshots[j] is MajorEmpireSnapshot)
                {
                    num4 += 1f;
                }
            }
            HeuristicFloat operand6 = default(HeuristicFloat);
            operand6.Add(num4, "Number of player");
            operand6.Add(operand, "Factor based on turn");
            HeuristicFloat result = default(HeuristicFloat);
            result.Add(1f, "Constant");
            result.Divide(operand6, "Factor based on number of player and turn.");
            return result;
        }

        private int[] turn = new int[]
		{
			0,
			20,
			50,
			100,
			150,
			200,
			250,
			300
		};

        private float[] factorForTurn = new float[]
		{
			16f,
			8f,
			3f,
			1f,
			0f,
			-0.5f,
			-0.65f,
			-0.75f
		};
    }
}
