﻿using System;
using Amplitude.AI.Core.AIPlayers.MajorEmpire;
using Amplitude.AI.Core.Brain.Allocators;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Brain.Strategic.Data;
using Amplitude.AI.Core.Brain.Strategic.Data.Diplomacy;
using Amplitude.AI.Core.Brain.Tasks.FleetMissionTasks.FrontBased;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Galaxy;

namespace Amplitude.AI.Core.Brain.Generators.Motherships
{
    public class LeechSystem : FrontBasedGenerator<MothershipHarvest, FrontMissionTask_Leech, GenerationData<Strategic.Data.Front, MothershipHarvest, FrontMissionTask_Leech>>
    {
        public override string Description
        {
            get
            {
                return "Send a leecher to a system.";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 5;
            }
        }

        public override string LocalizedName
        {
            get
            {
                return "Leech System";
            }
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<Strategic.Data.Front, MothershipHarvest, FrontMissionTask_Leech> taskContext)
        {
            return new HeuristicFloat(0.7f, "");
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, Strategic.Data.Front context, MothershipHarvest parent)
        {
            AIPlayer_MajorEmpire aiplayer_MajorEmpire = aiPlayer as AIPlayer_MajorEmpire;
            if (context.Enemy is MajorEmpireSnapshot && (aiPlayer.ControlledEmpireSnapshot.GetDiplomaticRelation(context.Enemy) as MajorDiplomaticRelationSnapshot).TurnsBeforeLeechingIsAllowed > 0f)
            {
                return;
            }
            for (int i = 0; i < context.AllAttackZones.Count; i++)
            {
                Zone zone = context.AllAttackZones[i];
                for (int j = 0; j < zone.GameNodes.Count; j++)
                {
                    GameNodeSnapshot gameNodeSnapshot = zone.GameNodes[j];
                    if (gameNodeSnapshot.IsLeechable && gameNodeSnapshot.HasColony(true, aiPlayer.ControlledEmpireIndex, true))
                    {
                        FrontMissionTask_Leech frontMissionTask_Leech = aiPlayer.Blackboard.FindFirstRunning<FrontMissionTask_Leech>(context.FrontGuid, gameNodeSnapshot.NodePosition);
                        if (frontMissionTask_Leech == null)
                        {
                            frontMissionTask_Leech = new FrontMissionTask_Leech(aiPlayer.Blackboard.GenerateGUID(), aiPlayer.ControlledEmpireSnapshot.GameEntityGuid, context.FrontGuid);
                            frontMissionTask_Leech.FrontGuid = context.FrontGuid;
                            frontMissionTask_Leech.TargetNode = gameNodeSnapshot.NodePosition;
                            frontMissionTask_Leech.InstanceAllocations = new ResourceAllocation[1];
                            frontMissionTask_Leech.InstanceAllocations[0] = new InstanceAllocation_Fleet(aiplayer_MajorEmpire.StrategicDataHolder.MilitaryData.LeecherPattern);
                            aiPlayer.Blackboard.Register<FrontMissionTask_Leech>(frontMissionTask_Leech);
                        }
                        base.ContinueDecision(aiPlayer, base.GetGenerationData(context, parent, frontMissionTask_Leech));
                    }
                }
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, Strategic.Data.Front context)
        {
            if (!aiPlayer.ControlledEmpireSnapshot.HasMotherships)
            {
                return;
            }
            AIPlayer_MajorEmpire aiplayer_MajorEmpire = aiPlayer as AIPlayer_MajorEmpire;
            if (aiplayer_MajorEmpire != null)
            {
                MajorDiplomacyRelationData majorDiplomacyRelationData = aiplayer_MajorEmpire.StrategicDataHolder.DiplomacyData.GetRelationDataByEmpireIndex(context.EnemyEmpireIndex) as MajorDiplomacyRelationData;
                if (majorDiplomacyRelationData != null && majorDiplomacyRelationData.IsCordialOrBetter)
                {
                    return;
                }
            }
            if (context.AllAttackZones.Count == 0)
            {
                return;
            }
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<MothershipHarvest>());
        }

        public LeechSystem()
        {
        }
    }
}
