﻿using System;
using Amplitude.AI.Common.DataStructure;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Brain.Generators.Science;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.ShipDesign;

namespace Amplitude.AI.Core.Brain.Generators.Military
{
    public class QueueImproveHullTechnology : ScienceBasedGenerator<ShipDesignSnapshot, Goal, GenerationData<ShipDesignSnapshot, Goal, NeedTechnology>>
    {
        public override string LocalizedName
        {
            get
            {
                return "Queue Improve Hull Technology";
            }
        }

        public override string Description
        {
            get
            {
                return "Queue Improve Hull Technology if we can afford them";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 0;
            }
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            MajorEmpireSnapshot majorEmpireSnapshot = aiPlayer.ControlledEmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot == null)
            {
                return;
            }
            for (int i = 0; i < aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableShipDesigns.Count; i++)
            {
                ShipDesignSnapshot shipDesignSnapshot = aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableShipDesigns[i];
                ShipDesignLayoutContextData layoutContextData = majorEmpireSnapshot.GetLayoutContextData(shipDesignSnapshot.Hull);
                if (layoutContextData != null)
                {
                    bool flag = false;
                    bool flag2 = true;
                    for (int j = 0; j < layoutContextData.Slots.Length; j++)
                    {
                        ShipLayoutSlotSnapshot shipLayoutSlotSnapshot = layoutContextData.Slots[j];
                        if (!shipLayoutSlotSnapshot.ArePrerequisitesValid)
                        {
                            flag = true;
                            if (shipLayoutSlotSnapshot.Costs.Costs != null)
                            {
                                for (int k = 0; k < shipLayoutSlotSnapshot.Costs.Costs.Length; k++)
                                {
                                    CostSnapshot costSnapshot = shipLayoutSlotSnapshot.Costs.Costs[k];
                                    if (Array.IndexOf<string>(service.GalaxySnapshot.StrategicNames, costSnapshot.ResourceName) >= 0 && majorEmpireSnapshot.GetResourceNet(costSnapshot.ResourceName) <= 0f)
                                    {
                                        flag2 = false;
                                    }
                                }
                            }
                        }
                    }
                    if (flag && flag2)
                    {
                        base.ContinueDecision(aiPlayer, shipDesignSnapshot);
                    }
                }
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, ShipDesignSnapshot context)
        {
            if (context.ShipRole == ShipPattern.ShipRole.MilitaryJuggernaut || context.ShipRole == ShipPattern.ShipRole.Battleship)
            {
                base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<UseJuggernaut>());
                return;
            }
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<ImproveCapabilities>());
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<ShipDesignSnapshot, Goal, NeedTechnology> generationData)
        {
            if (generationData.Parent is UseJuggernaut)
            {
                return new HeuristicFloat(1f, "");
            }
            return new HeuristicFloat(0.5f, "");
        }

        protected override bool IsValidConstructible(AIPlayer aiPlayer, ScienceImprovementData scienceImprovementData, ShipDesignSnapshot context, Goal parent)
        {
            if (context.CommandPoints == 1f && context.ShipRole == ShipPattern.ShipRole.Attacker)
            {
                return scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasCapability("TechnologyImproveHull1");
            }
            if (context.CommandPoints == 1f && context.ShipRole == ShipPattern.ShipRole.Support)
            {
                return scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasCapability("TechnologyImproveHull2");
            }
            if (context.CommandPoints == 3f && context.ShipRole == ShipPattern.ShipRole.Attacker)
            {
                return scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasCapability("TechnologyImproveHull3");
            }
            if (context.CommandPoints == 3f && context.ShipRole == ShipPattern.ShipRole.Support)
            {
                return scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasCapability("TechnologyImproveHull4");
            }
            if (context.ShipRole == ShipPattern.ShipRole.Carrier)
            {
                return scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasCapability("TechnologyImproveHull5");
            }
            if (context.ShipRole == ShipPattern.ShipRole.MilitaryJuggernaut)
            {
                return scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasCapability("TechnologyImproveHullJuggernaut");
            }
            return context.ShipRole == ShipPattern.ShipRole.Battleship && scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasCapability("TechnologyImproveHullBattleship");
        }

        public QueueImproveHullTechnology()
        {
        }
    }
}
